@@ -2,7 +2,7 @@
 // SPDX-License-Identifier: LGPL-2.1-or-later
 // Copyright (C) 2010-2024 celeron55, Perttu Ahola <celeron55@gmail.com>
 
-#include <algorithm>
+#include "servermap.h"
 
 #include "map.h"
 #include "mapsector.h"
@@ -16,18 +16,18 @@
 #include "profiler.h"
 #include "gamedef.h"
 #include "util/directiontables.h"
+#include "util/serialize.h"
 #include "rollback_interface.h"
 #include "reflowscan.h"
 #include "emerge.h"
-#include "mapgen/mapgen_v6.h"
 #include "mapgen/mg_biome.h"
 #include "config.h"
 #include "server.h"
+#include "serverenvironment.h"
 #include "database/database.h"
 #include "database/database-dummy.h"
 #include "database/database-sqlite3.h"
 #include "script/scripting_server.h"
-#include "irrlicht_changes/printing.h"
 #if USE_LEVELDB
 #include "database/database-leveldb.h"
 #endif
@@ -200,9 +200,9 @@ bool ServerMap::blockpos_over_mapgen_limit(v3s16 p)
 bool ServerMap::initBlockMake(v3s16 blockpos, BlockMakeData *data)
 {
 	assert(data);
-	s16 csize = getMapgenParams()->chunksize;
+	const v3s16 csize = getMapgenParams()->chunksize;
 	const v3s16 bpmin = EmergeManager::getContainingChunk(blockpos, csize);
-	const v3s16 bpmax = bpmin + v3s16(1, 1, 1) * (csize - 1);
+	const v3s16 bpmax = bpmin + csize - v3s16(1);
 
 	if (!m_chunks_in_progress.insert(bpmin).second)
 		return false;
@@ -279,10 +279,11 @@ void ServerMap::cancelBlockMake(BlockMakeData *data)
 }
 
 void ServerMap::finishBlockMake(BlockMakeData *data,
-	std::map<v3s16, MapBlock*> *changed_blocks, u32 now)
+	std::map<v3s16, MapBlock*> *changed_blocks, ServerEnvironment *env)
 {
 	assert(data);
 	assert(changed_blocks);
+	u32 now = env->getGameTime();
 	const v3s16 bpmin = data->blockpos_min;
 	const v3s16 bpmax = data->blockpos_max;
 
@@ -299,7 +300,13 @@ void ServerMap::finishBlockMake(BlockMakeData *data,
 		<< changed_blocks->size());
 
 	/*
-		Copy transforming liquid information
+		Process the chunk's liquid queue now.
+		This avoids sending many duplicate block updates.
+	 */
+	transformLiquidsLocal(*changed_blocks, data->transforming_liquid, env, g_settings->getS32("liquid_loop_max"));
+
+	/*
+		Copy remaining (if any) transforming liquid information
 	*/
 	while (!data->transforming_liquid.empty()) {
 		m_transforming_liquid.push_back(data->transforming_liquid.front());
@@ -771,7 +778,6 @@ MapBlock *ServerMap::loadBlock(const std::string &blob, v3s16 p3d, bool save_aft
 		if (!modified_blocks.empty()) {
 			MapEditEvent event;
 			event.type = MEET_OTHER;
-			event.low_priority = true;
 			event.setModifiedBlocks(modified_blocks);
 			dispatchEvent(event);
 		}
@@ -920,11 +926,10 @@ void ServerMap::transforming_liquid_add(v3s16 p)
 	m_transforming_liquid.push_back(p);
 }
 
-void ServerMap::transformLiquids(std::map<v3s16, MapBlock*> &modified_blocks,
-		ServerEnvironment *env)
+void ServerMap::transformLiquidsLocal(std::map<v3s16, MapBlock*> &modified_blocks, UniqueQueue<v3s16> &liquid_queue,
+		ServerEnvironment *env, u32 liquid_loop_max)
 {
 	u32 loopcount = 0;
-	u32 initial_size = m_transforming_liquid.size();
 
 	/*if(initial_size != 0)
 		infostream<<"transformLiquids(): initial_size="<<initial_size<<std::endl;*/
@@ -936,21 +941,18 @@ void ServerMap::transformLiquids(std::map<v3s16, MapBlock*> &modified_blocks,
 
 	std::vector<v3s16> check_for_falling;
 
-	u32 liquid_loop_max = g_settings->getS32("liquid_loop_max");
-	u32 loop_max = liquid_loop_max;
-
-	while (m_transforming_liquid.size() != 0)
+	while (!liquid_queue.empty())
 	{
 		// This should be done here so that it is done when continue is used
-		if (loopcount >= initial_size || loopcount >= loop_max)
+		if (loopcount >= liquid_loop_max)
 			break;
 		loopcount++;
 
 		/*
 			Get a queued transforming liquid node
 		*/
-		v3s16 p0 = m_transforming_liquid.front();
-		m_transforming_liquid.pop_front();
+		v3s16 p0 = liquid_queue.front();
+		liquid_queue.pop_front();
 
 		MapNode n0 = getNode(p0);
 
@@ -1026,7 +1028,7 @@ void ServerMap::transformLiquids(std::map<v3s16, MapBlock*> &modified_blocks,
 						// should be enqueded for transformation regardless of whether the
 						// current node changes or not.
 						if (nb.t != NEIGHBOR_UPPER && liquid_type != LIQUID_NONE)
-							m_transforming_liquid.push_back(npos);
+							liquid_queue.push_back(npos);
 						// if the current node happens to be a flowing node, it will start to flow down here.
 						if (nb.t == NEIGHBOR_LOWER)
 							flowing_down = true;
@@ -1221,15 +1223,15 @@ void ServerMap::transformLiquids(std::map<v3s16, MapBlock*> &modified_blocks,
 				// make sure source flows into all neighboring nodes
 				for (u16 i = 0; i < num_flows; i++)
 					if (flows[i].t != NEIGHBOR_UPPER)
-						m_transforming_liquid.push_back(flows[i].p);
+						liquid_queue.push_back(flows[i].p);
 				for (u16 i = 0; i < num_airs; i++)
 					if (airs[i].t != NEIGHBOR_UPPER)
-						m_transforming_liquid.push_back(airs[i].p);
+						liquid_queue.push_back(airs[i].p);
 				break;
 			case LIQUID_NONE:
 				// this flow has turned to air; neighboring flows might need to do the same
 				for (u16 i = 0; i < num_flows; i++)
-					m_transforming_liquid.push_back(flows[i].p);
+					liquid_queue.push_back(flows[i].p);
 				break;
 			case LiquidType_END:
 				break;
@@ -1238,7 +1240,7 @@ void ServerMap::transformLiquids(std::map<v3s16, MapBlock*> &modified_blocks,
 	//infostream<<"Map::transformLiquids(): loopcount="<<loopcount<<std::endl;
 
 	for (const auto &iter : must_reflow)
-		m_transforming_liquid.push_back(iter);
+		liquid_queue.push_back(iter);
 
 	voxalgo::update_lighting_nodes(this, changed_nodes, modified_blocks);
 
@@ -1247,6 +1249,15 @@ void ServerMap::transformLiquids(std::map<v3s16, MapBlock*> &modified_blocks,
 	}
 
 	env->getScriptIface()->on_liquid_transformed(changed_nodes);
+}
+
+void ServerMap::transformLiquids(std::map<v3s16, MapBlock*> &modified_blocks,
+		ServerEnvironment *env)
+{
+	// process the whole queue at most once, to rate-limit
+	u32 liquid_loop_max = std::min<u32>(m_transforming_liquid.size(), g_settings->getS32("liquid_loop_max"));
+
+	transformLiquidsLocal(modified_blocks, m_transforming_liquid, env, liquid_loop_max);
 
 	/* ----------------------------------------------------------------------
 	 * Manage the queue so that it does not grow indefinitely
